#copyright ReportLab Europe Limited. 2000-2016
#see license.txt for license details
__version__='3.3.0'

def _processLine(line, sep=',', conv=0):
    fields = []
    p = 0
    ll = len(line)
    ls = len(sep)
    line = line + ' '
    while (ll > 0 and (line[ll-1] in ' \012\015')): ll = ll-1

    while p < ll:
        #Skip unquoted space at the start of a field
        while p<ll and line[p]==" ": p = p + 1

        field = ""
        ql = 0
        while p < ll:
            #Skip unquoted space at the end of a field
            if ql == 0 and line[p] == " ":
                q = p
                while q < ll and line[q] == " ":
                    q = q + 1
                if q >= ll:
                    break
                elif line[q : q + ls] == sep:
                    p = q
            if ql == 0 and line[p : p + ls] == sep:
                break
            elif line[p] == '"':
                if ql == 0:
                    ql = 1
                elif line[p+1]=='"':
                    field = field+'"'
                    p = p + 1
                else:
                    ql = 0
            else:
                field = field + line[p]
            p = p + 1
        p = p + ls
        if conv:
            try:
                fields.append(int(field))
            except ValueError:
                try:
                    fields.append(float(field))
                except ValueError:
                    fields.append(field)
        else:
            fields.append(field)
    if line[ll-ls:ll]==sep:
        fields.append('')   #extra field when there's a separator at the end

    return fields

def read(fn,sep=',',conv=0):
    '''
    read the csv file fn and return the fields as a list of lists.
    sep is the separator to use and conv determines whether an attempt is made to convert
    numeric fields.
    '''
    from reportlab.lib.utils import open_for_read
    f = open_for_read(fn,'t')

    F = []
    for l in f.readlines():
        F.append(_processLine(l,sep,conv))

    if f is not fn: f.close()
    return F

class RowWrapper:
    def __init__(self,names,data):
        self.__dict__['_RowWrapper__names'] = names
        self.__dict__['_RowWrapper__data'] = data
    def __getattr__(self,a):
        return self.__data[self.__names[a]]
    def __setattr__(self,a,v):
        self.__data[self.__names[a]] = v
    def __getitem__(self,i):
        return self.__data[i]
    def __setitem__(self,i,v):
        self.__data[i] = v

class CSVWrapper:
    __wrapper = RowWrapper
    def __init__(self, rawNames, aNames, hdrRow, datarows):
        names = {}
        for i,n in enumerate(rawNames):
            try:
                names[aNames[i]] = hdrRow.index(n)
            except:
                raise ValueError('Cannot locate expected column header "%s"' % n)
        self.__names = names
        self.__rows = [self.__wrapper(names,row) for row in datarows]

    def __getitem__(self,i):
        return self.__rows[i]

    def __setitem__(self,i,v):
        if not isinstance(v,self.__wrapper): v = self.__wrapper(self.__names,v)
        self.__rows[i] = v

def modifyCSV(f,L):
    return list(map(lambda x,f=f:list(map(f,x)),L[:]))

def modifyCSVRows(f,L,R=[]):
    L=L[:]
    for r in R:
        L[r] = list(map(f,L[r]))
    return L

def modifyCSVCols(f,L,C):
    L=L[:]
    if C:
        for r in range(len(L)):
            for c in C:
                L[r][c] = f(L[r][c])
    return L

if __name__ == '__main__':
    from reportlab.lib.utils import getStringIO
    L=read(getStringIO('"abc""d,ef""ghi",23,34\n1,2,3\na,4,5\n6,c,d\n'))
    print('originally',L)
    def f(x):
        try:
            x = int(x)
        except:
            pass
        return x

    print('modifyCSV',modifyCSV(f,L))
    print('modifyCSVRows([1,3])',modifyCSVRows(f,L,[1,3]))
    print('modifyCSVCols([0,2])',modifyCSVCols(f,L,[0,2]))



__all__=(
        'getCaptcha',
        )
_allowed = 'ABCDEFGHJKLMNPQRTUVWYX3479'
_mx = len(_allowed)-1
def getCaptcha(n=5,fontName='Courier',fontSize=14,text=None,fillColor=None):
    '''return n random chars in a string and in a byte string structured
    as a GIF image'''
    from reportlab.graphics.shapes import Drawing, Group, String, \
        rotate, skewX, skewY, mmult, translate, Rect
    if not text:
        from random import randint, uniform
        text=''.join([_allowed[randint(0,_mx)] for i in range(n)])
    else:
        uniform = lambda l,h: 0.5*(l+h)
        n = len(text)
    baseline = 0
    x = 0
    G0 = Group()
    for c in text:
        x += 1
        A = translate(x,uniform(baseline-5,baseline+5))
        A = mmult(A,rotate(uniform(-15,15)))
        A = mmult(A,skewX(uniform(-8,8)))
        A = mmult(A,skewY(uniform(-8,8)))
        G = Group(transform=A)
        G.add(String(0,0,c,fontname=fontName,fontSize=fontSize))
        G0.add(G)
        x0,y0,x1,y1 = G0.getBounds()
        x = 1+x1
    G0.transform=translate(2-x0,2-y0)
    W = 4+x1-x0
    H = 4+y1-y0
    D = Drawing(W,H)
    if fillColor:
        from reportlab.lib.colors import toColor
        D.add(Rect(0,0,W,H, fillColor = toColor(fillColor),strokeColor=None))
    D.add(G0)
    return text, D.asString('gif')

if __name__=='__main__':
    text,g=getCaptcha()
    f=open('ci.gif','wb')
    f.write(g)
    f.close()
    print(text,'written to ci.gif')



#Autogenerated by ReportLab guiedit do not edit
from reportlab.lib.corp import RL_CorpLogo
from reportlab.graphics.shapes import Drawing, _DrawingEditorMixin
from reportlab.lib.colors import white, ReportLabBlue

class RL_LOGO(_DrawingEditorMixin,Drawing):
    def __init__(self,width=400,height=200,*args,**kw):
        Drawing.__init__(self,width,height,*args,**kw)
        self._add(self,RL_CorpLogo(),name='logo',validate=None,desc=None)
        self.logo.fillColor   = ReportLabBlue
        self.logo.background  = white
        self.logo.shadow      = .7
        self.height     = 86
        self.width      = 130
    def getContents(self):
        self.logo.width=self.width
        self.logo.height=self.height
        return Drawing.getContents(self)

if __name__=="__main__": #NORUNTESTS
    RL_LOGO().save(formats=['pdf'],outDir='.',fnRoot=None)



#Copyright ReportLab Europe Ltd. 2000-2016
#see license.txt for license details
'''release helper utility'''

import sys, os, time
from .buildutils import	do_exec, find_exe, getFlag, kill
from reportlab.lib.utils import getStringIO
from reportlab import rl_config

def _getArgs(f,args=None):
	m = f in args
	if m: args.remove(f)
	return m

def getNow():
	return time.strftime('%Y%m%d%H%M%S',time.gmtime(time.time()))

def _makeTag(tag,s,major,minor,st):
	return tag + s + s.join(map(str,[major,minor])) + (not st and s + getNow() or '')

class Releaser:
	tagfn= 'PROJECT_CVS_TAG', 'version.txt'
	def __init__(self,d,verbose=rl_config.verbose):
		assert os.path.isdir(d), 'Argument d="%s" isn\'t a directory!' % d
		self.cwd = os.getcwd()
		try:
			os.chdir(d)
			self.workingDir = os.getcwd()
		finally:
			os.chdir(self.cwd)
		self.verbose = verbose

	def goWorkingDir(self):
		os.chdir(self.workingDir)

	def resetDir(self):
		os.chdir(self.cwd)

	def _readtag(self,fn):
		try:
			return open(fn,'r').read().strip().split()[0]
		except:
			return None

	def __getattr__(self,name):
		if name=='tag':
			try:
				self.goWorkingDir()
				for fn in self.tagfn:
					tag = self._readtag(fn)
					if tag:
						self.tag = tag
						return tag
				raise ValueError('Can\'t locate tagfile in %s' % repr(self.tagfn))
			finally:
				self.resetDir()
		else:
			raise AttributeError("'%s' instance has no attribute '%s'" % (self.__class__.__name__, name))

	def _svn(self,args,fail=1):
		''' do a svn command and return the results '''
		svn = find_exe('svn')
		if type(args) is type([]): args = ' '.join(args)
		self.goWorkingDir()
		fout=getStringIO()
		do_exec(svn + ' ' + args,fout=fout,sys_exit=0,verbose=self.verbose>1,fail=fail)
		self.resetDir()
		T = fout.getvalue()
		T.replace('\r\n','\n')
		T.replace('\r','\n')
		return '\n'.join([x for x in T.split('\n') if not x or x[0]!='?'])

	def major_minor(self,t,s=None):
		if not s: s = t[len(self.tag)]
		x = t.split(s)[1:]
		if len(x[-1])>4: del x[-1]
		n = len(x)
		return list(map(int,(n>=1 and x[0] or 0, n>=2 and x[1] or 0)))

	def _tagFromArg(self,a):
		T = self._listTags(asList=1)
		if a is None:
			return T[-1]
		elif len(a)>len(self.tag):
			if a in T: return a
		else:
			# assume we have 0.0 or whatever
			for s in ('_','-','.'):
				if s in a:
					t = a.split(s)
					major_minor = int(t[0]), int(t[1])
					for t in T:
						if tuple(self.major_minor(t))==major_minor: return t
					break
		raise ValueError('%s is not a valid tag' % a)

	def _splitTag(self,a):
		for s in ('.','_','-'):
			if s in a:
				a = a.split(s)
				return '%s-%s-%s-%s' % (self.tag, a[0], a[1], getNow())
		return None

	def release(self,*args):
		'''
		find tag
		SVN update
		appmonitor.py dir writeSignedSourceManifest
		appmonitor.py dir verifySignedSourceManifest
		SVN tag
		appmonitor.py dir makeSourceArchive fullname
		'''
		args = list(args)
		nosvn = _getArgs('-nosvn',args)
		nvd = _getArgs('-n',args)
		zip = _getArgs('-zip',args)
		tgz = _getArgs('-tgz',args)
		tar = _getArgs('-tar',args)
		dt = _getArgs('-dt',args)
		n = len(args)
		if n==1:
			tag = self._splitTag(args[0])
		else:
			raise ValueError('release takes flags -n -nosvn -zip -tgz -tar tag')
		sTag = dt and tag or self.tag+'-%s-%s' % tuple(self.major_minor(tag))
		if self.verbose: print('Creating release files for '+sTag)
		if not nosvn:
			self._svn('up')
			if self.verbose: print('directory "%s" svn updated' % self.workingDir)
		from .appmonitor import AppMonitor
		a = AppMonitor(self.workingDir,verbose=self.verbose)
		a.writeSignedSourceManifest()
		if not a.verifySignedSourceManifest(): raise ValueError("Can't verify source_manifest")
		if nvd: copyDir = None
		else: copyDir = tag
		for go, kind in ((zip,'zip'),(tar,'tar'),(tgz,'tgz')):
			if go: a.makeSourceArchive('%s.%s' % (sTag,kind),kind=kind,copyDir=copyDir)

	def _getProject(self):
		try:
			self.goWorkingDir()
			root = open('CVS/Root','r').read().strip()
			repos = open('CVS/Repository','r').read().strip()
			if repos[0]=='/': repos = repos[len(root[root.find('/'):])+1:]
			return repos
		finally:
			self.resetDir()

	def getArchive(self,*args):
		args = list(args)
		fs = _getArgs('-fs',args)
		zip = _getArgs('-zip',args)
		tgz = _getArgs('-tgz',args)
		tar = _getArgs('-tar',args)
		dt = _getArgs('-dt',args)
		if not (fs or zip or tgz or tar): raise ValueError('getArchive needs an output format from -fs -tgz -tar -zip')
		n = len(args)
		if n>1: raise ValueError('getArchive takes at most one tag argument')
		if n and args[0]=='now':
			aTag = 'now'
			sTag = self.tag+'-now'
			act = '-D'
		else:
			aTag = self._tagFromArg(n and args[0] or None)
			sTag = dt and aTag or self.tag+'-%s-%s' % tuple(self.major_minor(aTag))
			act = '-r'
		xDir = os.path.join(self.workingDir,sTag)
		kill(xDir)
		self._cvs(['-z3 export','-d',sTag,act,aTag,self._getProject()])
		from .appmonitor import AppMonitor
		a = AppMonitor(xDir,verbose=self.verbose)
		a.writeSignedSourceManifest()
		if not a.verifySignedSourceManifest(): raise ValueError("Can't verify source_manifest")
		for go, kind in ((zip,'zip'),(tar,'tar'),(tgz,'tgz')):
			if go: a.makeSourceArchive('../%s.%s' % (sTag,kind),kind=kind,copyDir=None)
		if not fs:
			self.resetDir()
			self.goWorkingDir()
			kill(sTag)

if __name__=='__main__':
	def usage(msg=None):
		cmd = os.path.basename(sys.argv[0])
		if msg: print('%s error: %s' % (cmd, msg))
		print('       %s directory release [-n(oversiondir)] [-zip] [-tgz] [-tar] [-dt] tag' % cmd)
		print('       %s directory getArchive [-fs] [-zip] [-tgz] [-tar] [-dt] tag' % cmd)
		sys.exit(msg and 1 or 0)
	argv = sys.argv
	v = (getFlag('-v') and 1 or 0)+(getFlag('-v') and 1 or 0)

	if len(argv) < 2:
		usage()
	else:
		D = argv[1]
		if not os.path.isdir(D): usage('directory %s not found!' % D)

		name = argv[2]
		a = Releaser(D, verbose=v)
		m = getattr(a, name, None)
		if not m: usage('method "%s" not found!' % name)
		else:
			r = m(*argv[3:])
			if r: print(r)



from django.conf.urls.defaults import *
from . import views 


# Uncomment the next two lines to enable the admin:
# from django.contrib import admin
# admin.autodiscover()

urlpatterns = patterns('',
    # Example:
    # (r'^reportlab_rml_sample/', include('reportlab_rml_sample.foo.urls')),

    # Uncomment the admin/doc line below and add 'django.contrib.admindocs' 
    # to INSTALLED_APPS to enable admin documentation:
    # (r'^admin/doc/', include('django.contrib.admindocs.urls')),

    # Uncomment the next line to enable the admin:
    # (r'^admin/', include(admin.site.urls)),
    
    ('^$', views.search_form),
    (r'^getPDF/$', views.getPDF),

)



#Autogenerated by ReportLab guiedit do not edit
from d_horizontalbarchart import HorizontalBarChart
from reportlab.graphics.shapes import _DrawingEditorMixin
from rlextra.graphics.guiedit.datacharts import ODBCDataSource, DataAssociation, DataAwareDrawing, Array

class HorizontalBarChartDrawing(_DrawingEditorMixin,DataAwareDrawing):
	def __init__(self,width=400,height=200,*args,**kw):
		DataAwareDrawing.__init__(self,width,height,*args,**kw)
		self._add(self,HorizontalBarChart(),name='HorizontalBarChart',validate=None,desc='The main chart')
		self._fontName                  = 'Courier-Bold'
		self.height                    = 105
		self.width                     = 145
		self.dataSource                = ODBCDataSource()
		self.dataSource.sql            = 'SELECT chartId, name, value1, value2 FROM generic_bar'
		self.dataSource.groupingColumn = 0
		self.dataSource.associations   = Array(4, DataAssociation)
		self.dataSource.associations.element00 = DataAssociation(column=0, target='chartId', assocType='scalar')
		self.dataSource.associations.element01 = DataAssociation(column=1, target='HorizontalBarChart.categoryNames', assocType='vector')
		self.dataSource.associations.element02 = DataAssociation(column=[2], target='HorizontalBarChart.data', assocType='tmatrix')
		self.dataSource.associations.element03 = DataAssociation(column=3, target='HorizontalBarChart.labelRow2', assocType='vector')
		self.formats         = ['eps', 'pdf', 'gif', 'jpg','png']
		self.verbose         = 1
		self.outDir          = './output/'
		self.fileNamePattern = 'horizontalbarchart%03d'

if __name__=="__main__": #NORUNTESTS
	HorizontalBarChartDrawing().go()



#copyright ReportLab Europe Limited. 2000-2016
#see license.txt for license details
"""Source Obfuscation Utility

usage: subfusc.py file1.py file2.py file3.py

We often have config files containing things like
database user names and passwords.  On a web server
these would typically be compiled to bytecodes; but
a script-kiddie grepping through the source might find
exciting words like 'password'.  Furthermore, Python now
has a widely available and easy-to-use bytecode disassembler.
What's more, a customer has insisted that we encrypt such
files using a well known algorithm such as RC4.

This makes things one step harder.  Given a source file,
we generate an intermediate "obfuscated source file"
which is functionally equivalent but a lot less readable.
We then compile this to a pyc.  For the pyc to run, it must
have access to the subfusc module.

For greater security we could move the encrypt/decrypt functions
into C code in _rl_accel, and allow an optional seed to be passed
in.  Further refinements would involve signing the bytecodes.
Getting past this would require the ability to read binaries :-)

I tested this by running our test suite with an obfuscated
canvas.pyc and it all works.

Time for an example.  Let's say you start with this:
---config.py-----
breakfast='eggs'
lunch='spam'
dinner='beer'
------end---------

The function obfuscateModule('config.py') would return this string:

-----new module------
#obfuscated version of sample_raw.py
data = '5A9LKUj2z+lt36tyrFkJZp4H2vPGspAJRgrlhaV6emWOdGqP31wxNQghuulz'\
       'C3jFRZtRmF0kC3Bl\nfzyAhs7bjznaIdqtQvgxbHnys+U4LyInlNCe6MXI+h4'\
       'cuAqvXZv7rw/GOGEnQTtca1BX3bYynj/y\nnZNJGYrVYYfz1uCRwb6GM64Yxz'\
       'e2uor17IQHg163fMtKphHhD6hwj+OURDgCoKcLSQeOguyNexvJ\ninEZ58dMT'\
       'TZJNQ1UzZfQQ2WdBl5xnQjNcHFsHK6vBCRWx1k=\n'\

import subfusc
exec subfusc.loadByteCode(data)
del data
del subfusc
-----end----------------

The block of goop is actually a marshalled bytecode object,
run through various encryption routines.

The function obfuscompile('srcfile.py') will put the source above
in a temp file, compile it, and create 'srcfile.pyc' alongside
the original exactly as if Python had compiled it.  The script
handler calls this.


"""
__version__='3.3.0'
SEED = 'guido van rossum'
# substitute your desired algorithm module here.
# functions should take encode(text, key) and
# decode(text, key) as signatures.
import os
import sys
import base64
import marshal
import string
import tempfile
import py_compile
from rlextra.utils import rc4 

def lineWrap(stuff, length=60):
    "Wraps a string into list of strings with max length n chars"
    lines = []
    while len(stuff):
        head = stuff[0:length]
        tail = stuff[length:]
        lines.append(head)
        stuff = tail
    return lines


def obfuscateModule(srcFileName):
    """ this generates a runnable pyc file using a compiled
    code object in the "goop", encrypted."""
    output = []
    srcLines = open(srcFileName).readlines()
    if srcLines[0][0:2] == '#!':
        # this is a script, preserve
        output.append(srcLines[0].strip())
        srcText = string.join(srcLines[1:], '\n')
    else:
        srcText = string.join(srcLines, '\n')

    output.append('#obfuscated version of %s' % srcFileName)
    codeObj = compile(srcText, srcFileName, 'exec')
    marshalled = marshal.dumps(codeObj)
    encoded = rc4.encode(marshalled, SEED)
    asciified = base64.encodestring(encoded)
    wrapped = lineWrap(asciified)
    output.append('data = %s\\' % repr(wrapped[0]))
    for line in wrapped[1:]:
        output.append( '       %s\\' % repr(line))
    output.append('')
    output.append('from rlextra.utils import subfusc')
    output.append('exec subfusc.loadByteCode(data)')
    output.append('del data')
    output.append('del subfusc')
    return string.join(output, '\n')


def loadByteCode(textBlock):
    "Loads the compiled code.  This is put at the bottom of generated modules"
    unwrapped = string.join(string.split(textBlock))
    binary = base64.decodestring(unwrapped)
    marshalled = rc4.decode(binary, SEED)
    codeObj = marshal.loads(marshalled)
    return codeObj
    
def obfuscompile(srcFileName):
    """Turns a plain py file to a scrambled pyc next to it"""
    tempfilename = tempfile.mktemp()
    obfuscatedSource = obfuscateModule(srcFileName)
    open(tempfilename, 'w').write(obfuscatedSource)
    py_compile.compile(tempfilename, cfile=srcFileName + 'c', dfile=srcFileName)
    os.remove(tempfilename)

if __name__=='__main__':
    if len(sys.argv) < 1:
        print('usage: subfusc.py file1.py file2.py file3.py')
    else:
        for arg in sys.argv[1:]:
            obfuscompile(arg)
            print('compiled %sc' % arg)
        
    





import collections
#!c:/Python/python.exe
_SPACELEN=20
_SPACER= _SPACELEN*' '
def tryImport(name,vname=None,severity='Fatal'):
    try:
        m = __import__(name)
        f = getattr(m,'__file__','')
        if f: f = ' "%s"' % f
        if vname:
            if isinstance(vname, collections.Callable):
                v = vname()
            else:
                v = getattr(m,vname,'[unknown]')
            msg = 'version %s present%s.'  % (v, f)
        else:
            msg = 'present%s.'  % f
    except:
        msg = 'not present. %s!' % severity
    return '%s%s%s' % (name,max(_SPACELEN-len(name),1)*' ',msg) 
def main():
    import sys
    hexversion, platform, executable, exec_prefix, path = sys.hexversion, sys.platform, sys.executable, sys.exec_prefix, sys.path
    path = ('\n'+_SPACER).join(path)
    print('''
Python version:     %(hexversion)x
Python executable:  %(executable)s
Python exec prefix: %(exec_prefix)s
Python path:        %(path)s

Dependencies:''' % locals())

    def PIL_VERSION():
        from PIL import Image
        return Image.VERSION
    def Pmw_VERSION():
        from Pmw import version
        return version()
    for name, vname, severity in (
                                ('reportlab','Version','Fatal you need this'),
                                ('_rl_accel','version','Needed for speed'),
                                ('sgmlop',None,'Needed for paragraph speed'),
                                ('zlib',None,'Required for page compression'),
                                ('PIL',PIL_VERSION,'Required for PDF  & renderPM images.'),
                                ('_renderPM','_version','Required for bitmap image renderer.'),
                                ('pyRXP','version','Required for fast validating XML parser.'),
                                ('rlextra','Version','Fatal you need this'),
                                ('Pmw',Pmw_VERSION,'Required for the drawing editor'),
                                ('mx','__version__','Possibly required for datacharts'),
                                ):
        print(tryImport(name,vname,severity))


if __name__=='__main__':
    main()
